@@ -7,7 +7,7 @@ use crate::{
 use ahash::{AHashSet, AHasher};
 use serde_json::{Map, Value};
 
-use crate::paths::InstancePath;
+use crate::paths::{InstancePath, JSONPointer};
 use std::hash::{Hash, Hasher};
 
 // Based on implementation proposed by Sven Marnach:
@@ -82,12 +82,14 @@ pub(crate) fn is_unique(items: &[Value]) -> bool {
     }
 }
 
-pub(crate) struct UniqueItemsValidator {}
+pub(crate) struct UniqueItemsValidator {
+    schema_path: JSONPointer,
+}
 
 impl UniqueItemsValidator {
     #[inline]
-    pub(crate) fn compile<'a>() -> CompilationResult<'a> {
-        Ok(Box::new(UniqueItemsValidator {}))
+    pub(crate) fn compile<'a>(schema_path: JSONPointer) -> CompilationResult<'a> {
+        Ok(Box::new(UniqueItemsValidator { schema_path }))
     }
 }
 
@@ -101,16 +103,17 @@ impl Validate for UniqueItemsValidator {
         true
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         if self.is_valid(schema, instance) {
             no_error()
         } else {
             error(ValidationError::unique_items(
+                self.schema_path.clone(),
                 instance_path.into(),
                 instance,
             ))
@@ -118,20 +121,21 @@ impl Validate for UniqueItemsValidator {
     }
 }
 
-impl ToString for UniqueItemsValidator {
-    fn to_string(&self) -> String {
-        "uniqueItems: true".to_string()
+impl core::fmt::Display for UniqueItemsValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        "uniqueItems: true".fmt(f)
     }
 }
 #[inline]
 pub(crate) fn compile<'a>(
     _: &'a Map<String, Value>,
     schema: &'a Value,
-    _: &CompilationContext,
+    context: &CompilationContext,
 ) -> Option<CompilationResult<'a>> {
     if let Value::Bool(value) = schema {
         if *value {
-            Some(UniqueItemsValidator::compile())
+            let schema_path = context.as_pointer_with("uniqueItems");
+            Some(UniqueItemsValidator::compile(schema_path))
         } else {
             None
         }
@@ -139,3 +143,18 @@ pub(crate) fn compile<'a>(
         None
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::tests_util;
+    use serde_json::json;
+
+    #[test]
+    fn schema_path() {
+        tests_util::assert_schema_path(
+            &json!({"uniqueItems": true}),
+            &json!([1, 1]),
+            "/uniqueItems",
+        )
+    }
+}
